@@ -36,13 +36,13 @@ __all__ = [
     'DeliverySkeleton',
     'Maildir',
     'Mboxrd',
-    'MDA_qmaillocal',
     'MDA_external',
     'MDA_lmtp',
-    'MultiDestinationBase',
+    'MDA_qmaillocal',
     'MultiDestination',
-    'MultiSorterBase',
+    'MultiDestinationBase',
     'MultiSorter',
+    'MultiSorterBase',
 ]
 
 def exit_with(stderr,errmsg):
@@ -601,9 +601,9 @@ class MDA_external(DeliverySkeleton, ForkingBase):
 
       ignore_stderr (boolean, optional) - if set, getmail will not consider the
             program writing to stderr to be an error.  The default is False.
-            
+
       pipe_stdout (boolean, optional) - if set, stdout from external command
-            will be forwarded to stdout of getmail.  The default is True. 
+            will be forwarded to stdout of getmail.  The default is True.
     '''
     _confitems = (
         ConfInstance(name='configparser', required=False),
@@ -623,7 +623,7 @@ class MDA_external(DeliverySkeleton, ForkingBase):
         if not os.access(self.conf['path'], os.X_OK):
             raise getmailConfigurationError('%s not executable'
                                             % self.conf['path'])
-        if type(self.conf['arguments']) != tuple:
+        if not isinstance(self.conf['arguments'], tuple):
             raise getmailConfigurationError(
                 'incorrect arguments format; see documentation (%s)'
                 % self.conf['arguments']
@@ -670,10 +670,10 @@ class MDA_external(DeliverySkeleton, ForkingBase):
                 )
         except getmailOperationError as oe:
             raise getmailDeliveryError( 'delivery %s child operation error: %s' % (self, oe) )
-        
+
         if self.conf['pipe_stdout'] and child.out:
             self.log.info('%s\n' % (''.join(map(chr, child.out))))
-        
+
         self.log.debug('command %s %d exited %d\n'
                        % (self.conf['command'], child.childpid, child.exitcode))
         if child.exitcode:
@@ -912,12 +912,12 @@ class MultiSorterBase(MultiDestinationBase):
         try:
             _locals = self.conf['locals']
             # Special case for convenience if user supplied one base 2-tuple
-            if (len(_locals) == 2 and type(_locals[0]) == str
-                    and type(_locals[1]) == str):
+            if (len(_locals) == 2 and
+                isinstance(_locals[0], str) and isinstance(_locals[1], str)):
                 _locals = (_locals, )
             for item in _locals:
-                if not (type(item) == tuple and len(item) == 2
-                        and type(item[0]) == str and type(item[1]) == str):
+                if (not isinstance(item, tuple) and len(item) == 2 and
+                    isinstance(item[0], str) and isinstance(item[1], str)):
                     raise getmailConfigurationError(
                         'invalid syntax for locals; see documentation'
                     )
@@ -975,7 +975,7 @@ class MultiSorter(MultiSorterBase):
                ("jason@example.org", "/home/jasonk/Maildir/"),
                ("sales@example.org", "/home/karlyk/Mail/sales"),
                ("abuse@(example.org|example.net)", "/home/kellyw/Mail/abuse/"),
-               ("^(jeff|jefferey)(\.s(mith)?)?@.*$", "[jeff-mail-delivery]"),
+               ("^(jeff|jefferey)(\\.s(mith)?)?@.*$", "[jeff-mail-delivery]"),
                ("^.*@(mail.)?rapinder.example.org$", "/home/rapinder/Maildir/")
                )
 
